#!/bin/ash
#demo script for gtkdialog1 using speak as speach engine
#May_June2013 goingnuts
#2015: bugfix in record_me (fileextention). Run even if speak not present. 
#Message when record done. Check for lame. Message if filenames for save is empty.
#Show status (stopped/pause/playing). Delete wav file if mp3 created

#TODO:
#change status to stopped when play is done - how???

[[ -x ../../../src/gtkdialog1 ]] && PATH=../../../src:$PATH

#tested with espeak-1.46.02
[ ! $(which speak) ] && MISSING='<text><label>I need program speak to run...</label></text>'

#set defaults
lang="default"
voice="\"\""
html="false"
pitch=50
speed=175
volume=100
delay=10
#size of buttons
size=20
#relief used
relief="none"
echo "Status: Ready!" > /tmp/pmspeak_status

#load previous settings if present
if [ -f $HOME/.pmspeak ]; then
. $HOME/.pmspeak
fi

#create list of languages present
for X in $(ls /usr/share/espeak-data/voices/); do
	if [ -f /usr/share/espeak-data/voices/$X ]; then
		LANGS="$LANGS<item>$X</item>"
	fi
done

#create a list of variants present
for X in $(ls /usr/share/espeak-data/voices/!v/); do
	if [ -f /usr/share/espeak-data/voices/!v/$X ]; then
		VARS="$VARS<item>$X</item>"
	fi
done

#create a styleset
echo '
style "tooltips" {
  bg[NORMAL] = "#ffd000"
  fg[NORMAL] = "#000000"
  font = "-adobe-helvetica-medium-r-normal--*-120-*-*-*-*-*-*"
}

widget "gtk-tooltip*" style "tooltips"
' > /tmp/rcpmspeak

#and export it...
export RCFILE="/tmp/rcpmspeak"

#ash function file
echo '

save_settings () {
echo "lang=$LANGUAGE
voice=$VARIANT
html=$HTML
pitch=$PITCH
speed=$SPEED
volume=$VOLUME
delay=$DELAY
" > $HOME/.pmspeak
}

play_me () { 
#check status - are we playing?
if [ ! "$(pgrep speak)" = "" ]; then 
	kill -CONT $(cat mypid)
	echo "Status: Playing" > /tmp/pmspeak_status
	return	
else
	rm -f mypid
	#do not play empty text...
	if [ ! "${MYTEXT}" = "" ]; then
		[ "$HTML" = "true" ] && M="-m" || M=""
		speak ${M} -s $SPEED -a $VOLUME -p $PITCH -g $DELAY -v$LANGUAGE+$VARIANT "${MYTEXT}" &
		echo $! > mypid
		echo "Status: Playing" > /tmp/pmspeak_status
	fi
fi
}

pause_me () {
if [ -f mypid ]; then
   	speakPID=$(cat mypid)
	kill -STOP $speakPID
	echo "Status: Paused" > /tmp/pmspeak_status
fi
}	

stop_me () {
if [ -f mypid ]; then
    speakPID=$(cat mypid)
   	kill -CONT $speakPID
	kill $speakPID
	killall speak
	rm -f mypid
	echo "Status: Stopped" > /tmp/pmspeak_status
fi
}	

save_me () {
	#we only check if filename is empty - not if the content is
	if [ ! "${SAVEFILE}" = "" ]; then
		echo "${MYTEXT}" > ${SAVEFILE}
	else
		desktopbox "Missing filename - aborting..." & 2>/dev/null
		sleep 2
		MYPID=$(ps | grep NOTIFY_BOX | cut -d " " -f1-2 | tr -s " ")
		kill ${MYPID} 2>/dev/null
	fi
}

desktopbox () {
export NOTIFY_BOX="<wtitle>pmspeak</wtitle>
<vbox>
	<text>
		<label>${1}</label>
	</text>
</vbox>
"
gtkdialog1 --center --wmclass snsretro --program=NOTIFY_BOX &
}

record_me () {
	#dont bother if filename and content is empty
	if [ ! "${RECORDFILE}" = "" ] && [ ! "${MYTEXT}" = "" ]; then
		speak ${M} -s $SPEED -a $VOLUME -p $PITCH -g $DELAY -v$LANGUAGE+$VARIANT "${MYTEXT}" -w "${RECORDFILE}.wav" &
		#inform user that recording is in process
		desktopbox "Please wait - recording..." & 2>/dev/null
		while true; do
			sleep 1
			[ "$(pgrep speak)" = "" ] && break
		done
		MYPID=$(ps | grep NOTIFY_BOX | cut -d " " -f1-2 | tr -s " ")
		kill ${MYPID} 2>/dev/null	
		
		#check output formate
		if [ "${WAV}" = "false" ]; then
			lame -h --silent "${RECORDFILE}.wav" "${RECORDFILE}.mp3" &
			desktopbox "Please wait - converting to mp3..." & 2>/dev/null
			while true; do
				sleep 1
				[ "$(pgrep lame)" = "" ] && break
			done
			MYPID=$(ps | grep NOTIFY_BOX | cut -d " " -f1-2 | tr -s " ")
			kill ${MYPID} 2>/dev/null
			rm -f ${RECORDFILE}.wav
		fi	
		
		desktopbox "Recording finished!" & 2>/dev/null
		sleep 2
		MYPID=$(ps | grep NOTIFY_BOX | cut -d " " -f1-2 | tr -s " ")
		kill ${MYPID} 2>/dev/null
			
	else
		#filename or text is empty
		if [ "${RECORDFILE}" = "" ]; then
			desktopbox "Filename missing - aborting record" & 2>/dev/null
			sleep 2
		else
			desktopbox "No text to speak - aborting record" & 2>/dev/null
			sleep 2
		fi
		MYPID=$(ps | grep NOTIFY_BOX | cut -d " " -f1-2 | tr -s " ")
		kill ${MYPID} 2>/dev/null
	fi
}



' > /tmp/pmspeak_ash

#the save as GUI
export SAVE_AS='
<wtitle>pmspeak</wtitle>
<vbox>
	<hbox>
		<entry>
			<tooltip>Write filename to save...</tooltip>
			<variable>SAVEFILE</variable>
		</entry>
		<button>
			<input file stock="gtk-open"></input>
			<label>Select file</label>
			<tooltip>Browse filesystem...</tooltip>
			<action fileselection>SAVEFILE</action>
		</button>
	</hbox>
	<hbox>
		<button>
			<input file stock="gtk-save"></input>
			<label>Save</label>
			<tooltip>Save the file...</tooltip>
			<action>save_me</action>
			<action>closewindow:SAVE_AS</action>
		</button>
		<button cancel>
			<tooltip>Close without saving...</tooltip>
			<action>closewindow:SAVE_AS</action>
		</button>
	</hbox>	
</vbox>
'

#the record as GUI - check for lame
[ "$(which lame)" = "" ] &&	LAME='<visible>disabled</visible>'

export RECORD_AS='
<wtitle>pmspeak</wtitle>
<vbox>
	<hbox>
		<entry>
			<tooltip>Write filename to save...</tooltip>
			<variable>RECORDFILE</variable>
		</entry>
		<button>
			<input file stock="gtk-open"></input>
			<label>Select file</label>
			<tooltip>Browse filesystem...</tooltip>
			<action fileselection>RECORDFILE</action>
		</button>
	</hbox>
	<hbox>
		<radiobutton>
			<label>wav</label>
			<tooltip>Save as wav-file...</tooltip>
			<default>true</default>
			<variable>WAV</variable>
		</radiobutton>
		<radiobutton>
			<label>mp3</label>
			'$LAME'
			<tooltip>Save as mp3-file...</tooltip>
		</radiobutton>
	</hbox>
	<hbox>
		<button>
			<input file stock="gtk-save"></input>
			<label>Save</label>
			<tooltip>Save the file...</tooltip>
			<action>record_me</action>
			<action>closewindow:RECORD_AS</action>
		</button>
		<button cancel>
			<tooltip>Close without saving...</tooltip>
			<action>closewindow:RECORD_AS</action>
		</button>
	</hbox>	
</vbox>
'


#the GUI
export MAIN_DIALOG='
<wtitle>pmspeak</wtitle>
<vbox>
	'$MISSING'
	<hseparator></hseparator>
	# the action buttons
	<hbox>
		<button>
			<input file stock="gtk-new"></input>
			<label>New</label>
			<tooltip>Clear text and stop playing...</tooltip>
			<position>bottom</position>
			<height>'$size'</height>
			<width>'$size'</width>
			<default>'$relief'</default>
			<action clear>MYTEXT</action>
			<action>stop_me</action>
		</button>
		<button>
			<input file stock="gtk-open"></input>
			<label>Open</label>
			<tooltip>Choose a file to play...</tooltip>
			<position>bottom</position>
			<height>'$size'</height>
			<width>'$size'</width>
			<default>'$relief'</default>
			<action fileselection>MYTEXT</action>
		</button>
		<button>
			<input file stock="gtk-save-as"></input>
			<label>Save As</label>
			<tooltip>Save current text to a file...</tooltip>
			<position>bottom</position>
			<height>'$size'</height>
			<width>'$size'</width>
			<default>'$relief'</default>
			<action launchwindow>SAVE_AS</action>
		</button>
		<vseparator></vseparator>
		<button>
			<input file stock="gtk-media-record"></input>
			<label>Record</label>
			<tooltip>Record the current text to an audio file...</tooltip>
			<position>bottom</position>
			<height>'$size'</height>
			<width>'$size'</width>
			<default>'$relief'</default>
			<action launchwindow>RECORD_AS</action>
		</button>
		<vseparator></vseparator>
		<button>
			<input file stock="gtk-quit"></input>
			<label>Quit</label>
			<tooltip>Save current settings and quit program...</tooltip>
			<position>bottom</position>
			<height>'$size'</height>
			<width>'$size'</width>
			<default>'$relief'</default>
			<action>save_settings</action>
			<action>exit</action>
		</button>
	</hbox>
	<hseparator></hseparator>
	#the text area
    <frame Insert text to play>
        <edit>
        	<variable>MYTEXT</variable>
        	<tooltip>Type text or drag and drop a file here...</tooltip>
        </edit>
    
  	<hbox>
  		
    <button>
		<input file stock="gtk-media-play"></input>
		<tooltip>Play</tooltip>
		<action>play_me</action>
		<action>refresh:STATUSBAR</action>
	</button>
	<button>
		<input file stock="gtk-media-pause"></input>
		<tooltip>Pause</tooltip>
		<action>pause_me</action>
		<action>refresh:STATUSBAR</action>
	</button>
	<button>
		<input file stock="gtk-media-stop"></input>
		<tooltip>Stop</tooltip>
		<action>stop_me</action>
		<action>refresh:STATUSBAR</action>
	</button>
	
	</hbox>
	</frame>
	<notebook labels="Base settings|Advanced settings" show-tabs="yes" tabpos="top" >
	#base settings
	<vbox>
    	<hbox>
			<text>
				<label>Language:</label>
			</text>
			<combobox>
				<variable>LANGUAGE</variable>
				<tooltip>Select the language...</tooltip>
				<default>'$lang'</default>
				'$LANGS'
            </combobox>
		</hbox>
		<hbox>
			<text>
				<label>Voice:</label>
			</text>
			<combobox>
				<variable>VARIANT</variable>
				<tooltip>Select the voice or special effect to use...</tooltip>
				<default>'$voice'</default>
				'$VARS'
            </combobox>
        </hbox>
		<hbox>
            #if the -m option is used then HTML can be spoken directly.
            <checkbox>
				<label>HTML text</label>
				<tooltip>Check this if text contains HTML syntax...</tooltip>
				<default>'$html'</default>
				<variable>HTML</variable>
		</checkbox>
		</hbox>
	</vbox>
	<vbox>
    # Pitch adjustment, 0 to 99, default is 50
    	<hbox>
			<text>
				<label>Pitch:</label>
			</text>
			<hscale>
				<variable>PITCH</variable>
				<tooltip>Pitch adjustment...</tooltip>
				<default>'$pitch'</default>
				<mini>0</mini>
				<maxi>99</maxi>
				<height>30</height>
				<width>150</width>	
			</hscale>
		</hbox>
		# Amplitude, 0 to 200, default is 100
		<hbox>
			<text>
				<label>Volume:</label>
			</text>
			<hscale>
				<variable>VOLUME</variable>
				<tooltip>Sets the volume...</tooltip>
				<default>'$volume'</default>
				<mini>0</mini>
				<maxi>200</maxi>
				<height>30</height>
				<width>150</width>	
			</hscale>
		</hbox>
		# Speed in words per minute, 80 to 450, default is 175
		<hbox>
			<text>
				<label>Speed:</label>
			</text>
			<hscale>
				<variable>SPEED</variable>
				<tooltip>Speed in words per minute...</tooltip>
				<default>'$speed'</default>
				<mini>80</mini>
				<maxi>270</maxi>
				<height>30</height>
				<width>150</width>	
			</hscale>
		</hbox>
		
        # Word gap. Pause between words, units of 10mS at the default speed
		<hbox>
			<text>
				<label>Delay:</label>
			</text>
			<hscale>
				<variable>DELAY</variable>
				<tooltip>Pause between words (units of 10mS)...</tooltip>
				<default>'$delay'</default>
				<mini>0</mini>
				<maxi>100</maxi>
				<height>30</height>
				<width>150</width>	
			</hscale>
		</hbox>
    </vbox>
    </notebook>
    <entry>
		<variable>STATUSBAR</variable>
		<visible>Disabled</visible>
		<input>cat /tmp/pmspeak_status</input>
		#<action>file_me $ENTRY</action>
		#<action>refresh:EDIT</action>
	</entry>
</vbox>
'

gtkdialog1 --center --wmclass pmspeak --program=MAIN_DIALOG -i /tmp/pmspeak_ash

rm -f /tmp/pmspeak_ash status mypid /tmp/rcpmspeak /tmp/pmspeak_status
killall speak

wait

exit
